===================================================
Important
===================================================
Lenslet video to Multiview video Converter Version 1.0

Copyright: ONLY Available for ACADEMIC and MPEG Standardization Purposes
Production: Nagoya University

===================================================
Description
===================================================
Given a set of Raw video captured by multi-focused plenoptic camera (i.e. Ratrix camera,https://raytrix.de/) 
and its microlens calibration data, the provided source code can render multi-view video.

This tool is implemented by:

Fujii Laboratory
Department of Information and Communication Engineering
Graduate School of Engineering
Nagoya University

===================================================
Term of Use
===================================================
Term of Use: ANY kind of academic publication, and report using this tool MUST refer to the following TWO references.

[1] Shu Fujita, Sho Mikawa, Mehrdad Teratani(Panahpourtehrani), Keita Takahashi, and Toshiaki Fujii 
“Extracting Multi-View Images from Multi-Focused Plenoptic Camera,” 
International Workshop on Advanced Image Technology (IWAIT)/International Forum on Medical Imaging in Asia (IFMIA), 
in SPIE Digital Library, Singapore, January 2019

[2] Mehrdad Teratani(Panahpourtehrani), Shu Fujita, Wenzhe Ouyang, Keita Takahashi, Toshiaki Fujii, 
"3D Imaging System Using Multi-Focus Plenoptic Camera and Tensor Display," 
2018 International Conference on 3D Immersion (IC3D2018) in IEEE Xplore Digital Library, Brussels, Belgium, December 2018 

===================================================
Prerequisites 
===================================================
1. Boost
2. OpenCV 3.2.0 or above
We implemented our program on Ubuntu 18.04 LTS with g++ 7.3.0 compiler

===================================================
Instruction to run the program
===================================================
To build the software, execute the make command in the working directory.
$ make
A executable program, named 'converting' will be generated.

To demonstrate the software, we have included some test data capture by Raytrix R5, you can execute 'converting' 
with a parameter file.
$ ./converting param.cfg

When the program finished, it will automatically create a folder for storing the multi-view video.
Please make sure that you designate valid file paths.

Notes: 1. We recommend you to do a test with single frame input data firstly, while you set the parameters to default value. 
          If you find that the generated three lens-type images  (LensType_1.png, LensType_2.png and LensType_3.png) 
          are not arranged with default mode (lmode: FMN), please switch it to the suitable one.
          By single frame conversion, you may set the other parameters too.	
       2. Our software is designed for video processing, so we added frame number on input filename.
          In the case of that you want to process single image without frame number, please append a frame number to the filename.
       3. In the case of that input data is YUV420 format, please specify height and width of input data.
       4. LMC 1.0 is successfully tested for conversion of raw video captured by Raytrix R5 to multi-view video.


===================================================
Parameters
===================================================
viewNum            5
resolution_mode    1
pmode              0
mmode              2
lmode              1
Calibration_xml    .././Dataset/Test/R5_calib.xml
RawImage_Path      .././Dataset/Test/animals_Processed%03d.png
Output_Path        .././Output/Test/Result_%03d
input_model        0
output_model       0
start_frame        1
end_frame          1
height             2048
width              2048

===================================================
Parameter Description
===================================================
viewNum: Number of views at one direction. For example: viewNum = 5 means 5x5 views in 2D configuration
         Currently, we recommend you to set this value to 5 or 7 to ensure good rendering quality from Lenslet video to Multi-view Video.
         viewNum = 5 (default value) 5x5 views in 2D configuration
         viewNum = 7                 7x7 views in 2D configuration

resolution_model: This value must be a integer. It is corresponded to the "subImageOverlap" in the "MultiViewImage.cpp".
                  (for Raytrix R5)
                  Resolution mode = 1: 912x880 (default value)
                  Resolution mode = 2: 832x792
                  Resolution mode = 3: 744x712
                  Resolution mode = 4: 656x624
                  Resolution mode = 5: 568x544
                  Resolution mode = 6: 480x456

pmode: patch size estimation mode. Please refer to our paper [1][2] for more details.
       patchEstimationMode, 
       LAPLACIAN_BASE_ALL = 0, (default value) calculating Laplacian sum using all views,
       LAPLACIAN_BASE_CENTER = 1, calculating Laplacian sum using center view only,

       There are two patch size estimation method using depth information"
       DEPTH_BASE_V1 = 2, 
       DEPTH_BASE_V2 = 3,     
       Please refer following paper for the details,
       Ref: T. Georgiev and A. Lumsdaine, "Reducing Plenoptic Camera Artifacts", Computer Graphics forum, 2010.

mmode: merge mode for different lens type. Please refer to our paper for more details.
       mergeMode 
	   AVERAGE = 0    merge different lens type using arithmetic mean,
       GAUSSIAN = 1   merge different lens type using Gaussian weights,
       BLEND = 2     (default value), merge different lens type using arithmetic mean and Gaussian weights both.

lmode: lens Alignment mode: 
       NMF = 0, (N = LensType_1.png, M = LensType_2.png and F = LensType_3.png)
       FMN = 1, (F = LensType_1.png, M = LensType_2.png and N = LensType_3.png) (default)
       NFM = 2, (N = LensType_1.png, F = LensType_2.png and M = LensType_3.png)
       FNM = 3  (F = LensType_1.png, N = LensType_2.png and M = LensType_3.png)
	   
       This sets the alignment order of three micro-lens type. 
       NMF means {(N)ear, (M)iddle, (F)ar} focal distances for {LensType_1.png, LensType_2.png and LensType_3.png}, respectively.
       For example: NMF represents that the generated lenstype should be arranged with Near focal distance, Middle focal distance, Far distance.

       We recommend you to do a test with single frame input data firstly, while you set the parameters to default value. 
       If you find that the generated three lens-type images  (LensType_1.png, LensType_2.png and LensType_3.png) 
       are not arranged with default mode (lmode: FMN), please switch it to the suitable one.
       By single frame conversion, you may set the other parameters too.	   

Calibration_xml: The directory of camera calibration file and the file name.

RawImage_Path: The directory of  lenslet raw images and their file name format.

Output_Path: The directory for output multi-view video

input_model: 0 or 1; 0 means RGB model, 1 means YUV420 model

output_model: 0 or 1; 0 means RGB model, 1 means YUV420 model

start_frame: To set the start frame number of input data. 

end_frame: To set the end frame number of input data. 